/* Golem Project for Shannon Chappel
May 8 2014, post expo coding, getting ready for maker faire
 Double Rainbow 2 assembly, responsible for:
	- Receiving remote control signals
		- Chest gear - INPUT D4
		- Weapon activation - INPUT D7
	- Driving the following:
		- Weapon motor driving - FET D3 / Channel G1
		- Chest gears (feature #3) - FET D6
		- signaling weapon core to activate (feature 17, on own '328 circuit) - OUTPUT D8 & D13 (for wrist lights)
		- Head systems, consisting of: - FET D5
			- Head Tubes (feature 14)
			- Neck Tubes (feature 13)
			- Collar letters (feature 1)
			- mouth lighting (feature 11)
		- Center torso Cell back light (feature 5) - FET D9 (shared)
		- Groin tube lighting (feature 6) - FET D9 (shared)
		- Jaw servo signal (another Ardweeny signaled from D12)

	- Signaling
		- Start animation condition to weapon core (active low on D8)
		- Start animation condition to weapon arm tubes (active low, same cct as above)
		
		
Released under a Creative-Commons, Attribution, Share-alike License 4.0
Dave Hrynkiw
Solarbotics Ltd, April 2014

*/

// Remote setup
#define remotePin1 4 // input D4 for activating chest gears
#define remotePin0 7 // input D7 for controlling weapon activation  

// Weapon motor setup
#define weaponMotorPin 3 //output D3 controls FET "G1", and has been HACKED TO ADD BACK EMF DIODE (so don't change this channel)
#define potPin 0		// temporary for using trimpot to adjust speed
int weaponMotorMax=165; //set a limit to the motor speed. Using tests, 165 was max speed. Signal is inverted so lower=faster. 255-165=90
//int weaponMotorStep=(255-weaponMotorMax)/8;
int weaponMotorSpeed=255; //weapon speed initializer
//int weaponMotorMax = 1;

int remote0 = 0; //Initialize remote button state variables
int remote0DelayCycle = 100; //mS to scale up/down operation
int remote0GrowFactor = 1;	//incrementation variable
int remote0GrowLimit = 4;  // Maximum limit
int remote1 = 0;	

//ignore above crap. This is the stuff to get the weapon moving
unsigned long lastMillisWeapon;
unsigned long weaponTimeout = 250; //quarter second weapon debouncer run period
boolean weaponRunFlag = false;
#define pin13 13


// Chest gear setup
#define chestGearPin 6 // output D6 controls FET "B1"
int chestGearSpeedMax = 255; //Maximum power to feed the gears in the chest (out of 254)
unsigned long lastMillisGears;
unsigned long gearsTimeout = 20000; //20 seconds on time for each press
boolean gearsRunFlag = false;

// Weapon Core signaling
#define weaponCorePin 8 //output D8  Digital control pin to the core lightshow
#define weaponWristPin 12 // output D12 for arm tube signals

// Head Systems on D5  
#define headSystemsPin 5 //output controls FET "R1"
boolean breakHead = false;
#define headBreakerPin A0
#define headSystemsMaxBrightness 155	//Maximum head brightness

// Center Torso & groin
#define torsoGroinPin  9 //output controls FET "B2"
#define torsoGroinMaxBrightness 155 //Maximum brightness


void setup()
{
	pinMode(remotePin0, INPUT);
	pinMode(remotePin1, INPUT);
	pinMode(pin13, OUTPUT);
	pinMode(weaponCorePin, OUTPUT);
	pinMode(weaponMotorPin, OUTPUT);
	pinMode(potPin, INPUT);
	pinMode(chestGearPin,OUTPUT);
	pinMode(headSystemsPin, OUTPUT);
	pinMode(torsoGroinPin, OUTPUT);
	pinMode(headBreakerPin, INPUT_PULLUP);

	//initialize the port values
	digitalWrite(weaponCorePin, HIGH); //Activate on LOW (so, start off
	digitalWrite(weaponMotorPin, HIGH); //Weapon motor OFF
	digitalWrite(chestGearPin, LOW);	//chest gear off
	digitalWrite(headSystemsPin, LOW);	//head circuits off
	digitalWrite(torsoGroinPin, LOW);  //Groin & cell backlights off

}

void loop(){
//weaponTest();	//Use a trimpot to check motor speed

 if (digitalRead(headBreakerPin) == LOW) {breakHead = true;} else {breakHead = false;}

weaponMotor();		//Weapon spin up function from remote
chestGear();		// Chest gear spin up function from remote
headSystems();
torsoGroin();
//headJaw();

}
void chestGear(){
	if (digitalRead(remotePin0) == HIGH) {
		gearsRunFlag=true;
		lastMillisGears=millis();
		digitalWrite(chestGearPin, 1); //Turn it on
	}
	if (timeSince_ms (lastMillisGears)>gearsTimeout){
		gearsRunFlag=false;
	}
	if (gearsRunFlag) {
		digitalWrite(chestGearPin, 1); //Turn it on
	}
	else{
		digitalWrite(chestGearPin, 0); //if nothing else, turn it off.
	}
}
void weaponMotor(){

	if (digitalRead(remotePin1) == HIGH) {
		weaponRunFlag=true;
		lastMillisWeapon=millis();
		analogWrite(weaponMotorPin, weaponMotorMax); //Turn it on
		digitalWrite(weaponCorePin, 0);				// Send the signal to the wrist and weapon lightshow (enable LOW)
		digitalWrite(weaponWristPin, 0);
		digitalWrite(pin13,1);						//debug LED
	}
	else {
	if (timeSince_ms (lastMillisWeapon)>weaponTimeout){  // Did it timeout? If so, shut'er down
		weaponRunFlag=false;
	}
	}
	if (weaponRunFlag) {
		analogWrite(weaponMotorPin, weaponMotorMax); // Check to see if it should still be on
		digitalWrite(weaponCorePin, 0);
		digitalWrite(weaponWristPin, 0);
	}
	else{
		digitalWrite(weaponMotorPin, 1); //if nothing else, turn it off.
		digitalWrite(weaponCorePin, 1);
		digitalWrite(weaponWristPin, 1); 
		digitalWrite(pin13, 0);
	}
}
/*void weaponMotor(){
	if (digitalRead(remotePin1)== HIGH){						// Is the button pressed, ramp the speed up
		analogWrite(weaponMotorPin, weaponMotorMax);  //Send the update to the pin.
	}
	else
	{
		analogWrite(weaponMotorPin,255); //Turn it off
	}
}
*/

/*
(void weaponMotor(){
	if (digitalRead(remotePin1)== HIGH){						// Is the button pressed, ramp the speed up
		digitalWrite(weaponCorePin, 0);							// Signal lightshow to start in both core and arm tubes (active LOW)
		if(millis() - lastMillisWeapon > remote0DelayCycle) {
			if (remote0GrowFactor<remote0GrowLimit) {
				weaponMotorSpeed=255-(remote0GrowFactor*weaponMotorStep); // Step the value down from 255
				weaponMotorSpeed=max(weaponMotorSpeed, weaponMotorMax); // Make sure the maximum motor speed is no smaller than weaponMotorMax
				++remote0GrowFactor;
			}
			lastMillisWeapon=millis();
		}
			
	}
	else // if button release, turn off the motor, NOW (no ramp-down)
	{
		weaponMotorSpeed=255;
		digitalWrite(weaponCorePin, 1); //Turn off weapon core / arm tube signal (active LOW)
		remote0GrowFactor = 1;
	}
	analogWrite(weaponMotorPin, weaponMotorSpeed);  //Send the update to the pin.

}	)
*/ 


void headSystems(){
	if (breakHead==false) {  //breaker condition
		analogWrite(headSystemsPin, 0); //Turn it off
	}
	else
	{
		analogWrite(headSystemsPin, headSystemsMaxBrightness); //Turn it on
	}
}

void torsoGroin(){
	if (breakHead==true){  //Use same breaker switch as for head
		analogWrite(torsoGroinPin, 0); //Turn it off
	}
	else
	{
		analogWrite(torsoGroinPin, torsoGroinMaxBrightness); //turn it on
	}
}

#ifdef TESTMODE
void weaponTest()
{
	int val = analogRead(potPin);
	val = map(val, 0, 1023, 0, 254);
	analogWrite(weaponMotorPin, val);
}
#endif

unsigned long timeSince_ms( unsigned long tcomp){						//Used for candle flicker effect in the body crystals
	// This function deals with wrap-around on the millis() clock.
	// It can only handle a single wrap, of course
	unsigned long tnow = millis();
	
	if( tnow < tcomp)
	return( tnow + 4294967295 - tcomp);
	
	return( tnow - tcomp);
}