/*Golem Weapon Core lighting array
Uses some clever code from blinkenlights to get following fade from a Larsonlike animation
Dave Hrynkiw Solarbotics Ltd.
March 2014

*/

//Assign ring levels to appropriate I/O line
#define r1 3
#define r2 4
#define r3 5
#define r4 6
#define r5 7
#define r6 8
#define r7 9
const int ledRings[] = {r1, r2, r3, r4, r5, r6, r7};  //Set me up the list of pins to be used with the LEDs
int ringCount = 7; // Number of elements we're USING in array (0 to 6)
int lastMillis = 0;
int delayCycle = 2000; //delay between changes in speed
int growMax = 8; // Constant to fix maximum speed and brightness of the pulse
int growFactor = 1; //Start point. Must be integer minimum of 1
int brightBaseline = 2;

void setup()
{
	for (int thisRing = 0; thisRing < ringCount; thisRing++) {
		pinMode(ledRings[thisRing], OUTPUT);
	}
}

byte brightness(const int led, const int pos) {  //Test each LED for distance from focused position. The further way it is, ramp the brightness down
	switch (pos-led) {
		case 0:     return 32;					// LED IS the Pos'n? Make it full bright (32/32)
		case 1:     return 25;					// 1 LED away from Pos'n? 
		case 2:     return 12;					// 2 LEDs away?
		default:    return brightBaseline;
											// Even further away? Just make it barely on
	}
}

/*
uint8_t brightness(const int8_t led, const int8_t pos) {
	switch (min(abs(led-pos),abs(6-led-pos))) {
		case 0:     return 32;
		case 1:     return 16;
		case 2:     return 6;
		case 3:     return 1;
		default:    return 1;
	}
}
*/

void pulse_width_modulation(const byte position) {   //Position is the present focused LED location
	for(byte times=0; times<(80/growFactor) ; ++times) {							// Loop "times" higher for a slower action
		for (byte brightLevel=0; brightLevel<32; ++brightLevel) {	// Start of "fake" PWM. Looping thru all 32 levels of brightness that are available, from off (0) to full on (32)
			for (int led=0; led<ringCount; ++led) {					// Check each LED against this level of brightness. Is it supposed to be on?
				digitalWrite(ledRings[led], (brightness(led, position) > brightLevel)); //If tested brightness value is greater than brightLevel, turn it on
			}
		}
	}
}

void loop() {

  if (millis() - lastMillis > delayCycle) {  //check to see if it's time to change the speed delay
	  if (growFactor < growMax) {
		++growFactor;
		brightBaseline=2*growFactor;
		}
	  lastMillis = millis();
	  
  }

	static int pos=0;
	
	while(pos<10) {
		pulse_width_modulation(pos);
		++pos;
	}
	pos=0;
/*
	while(pos>0) {
		--pos;
		pulse_width_modulation(pos);
	}
*/
}

/*
Lets analyze the sketch.
The underlying idea is that the glowing effect is some kind of cursor that has a position that moves back and forth.
This is what the main loop takes care of.

The brightness function will compute the desired brightness of an LED depending on the LEDs number and the position
of the cursor.

In between is the pulse_width_modulation. This function will cycle 10 times through 32 phases.
For each phase it will cycle through all LEDs.
Depending on any LEDs desired brightness and the phase it will switch the LED on and off.

For example: Suppose the brightness function returns 16 for an LED.
In this case the LED will be on for phase in 0..15 and off for phase in 16..31.
Thus the LED would be on half of the time. Accordingly it will be visibly on but dimmer as an LED that is always on.
*/